-- -----------------------------------------------
-- CB Trade Chat Filter (TCF)
-- CB_TradeChatFilter.lua
-- Author: ChuckB
-- (c) 2010, 2012, 2013 ChuckB, All rights reserved.
-- -----------------------------------------------

TCF = CreateFrame("Frame")
TCF:RegisterEvent("ADDON_LOADED")
TCF:SetScript("OnEvent", function(self, event, ...) if TCF[event] then return TCF[event](self, event, ...) end end)
TCF:Show()

function TCF:ADDON_LOADED(event)
	TCF_Settings = TCF_Settings or {}
	TCF_Settings.tradeChatFilter = TCF_Settings.tradeChatFilter or true
	TCF_Settings.FrenchFilter = TCF_Settings.FrenchFilter or true
	TCF_Settings.debug = TCF_Settings.debug or false 

	-- Filter trade channel messages
	--    For those players < 80, the following typical LFG type msgs in TRADE channel
	--    are nothing but 'noise' if not plain annoying.  So, filter them out.
	--    We filter arena team requests (2v2, 3v3, 5v5), LFG msgs, and various 80 raid instances
	--    Note:  regex expressions could be added but I wanted to keep this very SIMPLE
	--           in case anyone wanted to update the list on their own.
        -- Specify as lower case only
	TCF_TradeFilterList = TCF_TradeFilterList or {
	"2v2",
	"3v3",
	"5v5",
	"lf1m",
	"lf2m",
	"lfm",
	"lfg",
	"icc",
	"icc10",
	"icc25",
	"icecrown citadel",
	"toc",
	"togc",
	"voa",
	"voa10",
	"voa25",
	"wg",
        "naxx",
        -- Level 80-90 related messages to filter
        "mv",
        " sha", "sha ",   -- to avoid matching real words containing sha (shatt, shard, etc)
        "rbg"  -- rated battle ground
	}

        -- French language filter
        -- Specify as lower case only
	TCF_FrenchLanguageFilterList = TCF_FrenchLanguageFilterList or {
        "bonjour", "guilde", "est", "nomme", "cest", "bien", "peux", "avance",
            -- From french.about.com Top 100 French words
        "le", "etre", "avoir", "de", "un", "une", "des", "je", "il", "ils", "ce", "pas",
        "et", "vous", "ca", "que", "ne", "faire", "qui", "oui", "alors",
        "mais", "elle", "elles", "en", "le", "la", "les", "du", "des", "dans",
        "se", "aller", "au", "aux", "bien", "ce", "cet", "cette", "ces", "tu", "la", "comme", "voir"
            -- there are more on the web page...but let us start with the above
        }
end -- of function TCF:ADDON_LOADED()

-- Simple help function that displays basic help text to the user on slash command help
local function TCF_help()
	print("/tf or /tcf enable | on | disable | off | show | help | debugon | debugoff")
	print("enable or on == turns on the filtering")
	print("disable or off == turns off the filtering basically giving you the default Blizzard behavior")
	print("show == shows the frame for configuration")
	print("help == shows this help text")
	print("debugon == turns on the debugging - shows the messages that are being filtered")
	print("debugoff == turns off the debugging messages")
	print("Thanks for trying the AddOn!  Hope you enjoy it!!")
end

-- Various ways to access the addon via slash commands
SLASH_TCF1 = "/tcf";
SLASH_TCF2 = "/tf";
SLASH_TCF3 = "/tradechatfilter"
SLASH_TCF4 = "/tradefilter"

SlashCmdList["TCF"] = function(msg)
	if (msg == "enable" or msg == "on") then
		print("trade chat filter: enabled")
		TCF_Settings.tradeChatFilter = true
		TCF_Settings.FrenchFilter = true
	elseif (msg == "disable" or msg == "off") then
		print("trade chat filter: disabled")
		TCF_Settings.tradeChatFilter = false
	elseif msg == "debugon" then
		print("trade chat filter: debug enabled!!  Use debugoff option to turn off debugging.")
		TCF_Settings.debug = true 
	elseif msg == "debugoff" then
		print("trade chat filter: debug disabled")
		TCF_Settings.debug = false
	elseif msg == "show" then
		TCF_Options:Show()
--		print("FYI:  you can press ESC to close the TCF window")
	elseif msg == "help" then
		TCF_help()	
	else	
	   -- If we got an option on the slash command we didn't recognize above, just display help
	   TCF_help()
	end
end

-- Frame local variables
local f		-- Main Frame
local titlestr 	-- Font String for Filter title
local ts 	-- Font String
local ts2 	-- Font String for French
local tb0  	-- Frame for Filter title 
local tb  	-- Check Button/Box
local tb2  	-- Check Button/Box for French
local closebtn	-- Button

-- Main Frame
f = CreateFrame("Frame", "TCF_Options", UIParent)
	f:SetWidth(240)
	f:SetHeight(120)
	f:SetPoint("CENTER", UIParent, "CENTER")
	f:SetBackdrop({
		bgFile = "Interface\\CHATFRAME\\CHATFRAMEBACKGROUND",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = "true",
		tileSize = 32,
		edgeSize = 10,
		insets = {left = 3, right = 3, top = 3, bottom = 3}
		})
	-- f:SetBackdropColor(.25,.1,.25)
	f:SetBackdropColor(.30,.2,.30)
	f:SetBackdropBorderColor(0,.5,0)
        -- make frame draggable/movable
        f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", f.StopMovingOrSizing)
	tinsert(UISpecialFrames,f:GetName())

-- Font String for text of title 
titlestr = f:CreateFontString("FilterTitleText")
	titlestr:SetFontObject("GameFontGreenSmall")
	titlestr:SetJustifyH("LEFT")
	titlestr:SetWidth(190)
	titlestr:SetText("Filter Trade/General Chat Options")
	titlestr:SetPoint("TOPLEFT", TCF_Options, "TOPLEFT", 5,-10)

-- Frame for Filter text title 
tb0 = CreateFrame("Frame", "TCF_OptionTitle", TCF_Options, UIParent)
		tb0:SetHitRectInsets(0,0,0,0)
		tb0:SetPoint("LEFT", FilterTitleText, "RIGHT", 10, 0)
		tb0:SetScale(.75)

-- Font String for text of filter trade channel 
ts = f:CreateFontString("TradeChatOptionText")
	ts:SetFontObject("GameFontGreenSmall")
	ts:SetJustifyH("LEFT")
	ts:SetWidth(190)
	ts:SetText("Remove level 80-90 messages?")
	ts:SetPoint("TOPLEFT", TCF_Options, "TOPLEFT", 5,-50)

-- Check box/button for filtering trade chat 
tb = CreateFrame("CHECKBUTTON", "TCF_TradeChatCheckBox", TCF_Options, "OptionsCheckButtonTemplate")
		tb:SetHitRectInsets(0,0,0,0)
		tb:SetPoint("LEFT", TradeChatOptionText, "RIGHT", 10, 0)
		tb:SetScale(.75)
		tb:SetScript("OnClick", function(self)
			if self:GetChecked() == 1 then 
				TCF_Settings.tradeChatFilter = true
				if TCF_Settings.debug then
					print("<OnClick> tcf set to true") 
				end
			else 
				TCF_Settings.tradeChatFilter = false
				if TCF_Settings.debug then
					print("<OnClick> tcf set to false") 
				end
			end 
		end)
		tb:SetScript("OnShow", function(self)
			if TCF_Settings.debug then
				if TCF_Settings.tradeChatFilter == true then 
					print("<Show> tcf set to true") 
				else
					print("<Show> tcf set to false") 
				end
			end
			if TCF_Settings.tradeChatFilter == true then 
				self:SetChecked(true)
				if TCF_Settings.debug then
					print("<Show> setChecked(true)") 
				end
			else 
				self:SetChecked(false)
				if TCF_Settings.debug then
					print("<Show> setChecked(false)") 
				end
			end 
		end)

-- Font String for text of French Filter text 
ts2 = f:CreateFontString("FrenchFilterOptionText")
	ts2:SetFontObject("GameFontGreenSmall")
	ts2:SetJustifyH("LEFT")
	ts2:SetWidth(100)
	ts2:SetText("Remove French?")
	ts2:SetPoint("TOPLEFT", TCF_Options, "TOPLEFT", 5, -70)

-- Check box/button for filtering trade chat 
tb2 = CreateFrame("CHECKBUTTON", "TCF_FrenchLanguageCheckBox", TCF_Options, "OptionsCheckButtonTemplate")
		tb2:SetHitRectInsets(0,0,0,0)
		tb2:SetPoint("LEFT", FrenchFilterOptionText, "RIGHT", 10, 0)
		tb2:SetScale(.75)
		tb2:SetScript("OnClick", function(self)
			if self:GetChecked() == 1 then 
				TCF_Settings.FrenchFilter = true
				if TCF_Settings.debug then
					print("<OnClick> french filter set to true") 
				end
			else 
				TCF_Settings.FrenchFilter = false
				if TCF_Settings.debug then
					print("<OnClick> french filter set to false") 
				end
			end 
		end)
		tb2:SetScript("OnShow", function(self)
			if TCF_Settings.debug then
				if TCF_Settings.FrenchFilter == true then 
					print("<Show> french filter set to true") 
				else
					print("<Show> french filter set to false") 
				end
			end
			if TCF_Settings.FrenchFilter == true then 
				self:SetChecked(true)
				if TCF_Settings.debug then
					print("<Show> setChecked(true)") 
				end
			else 
				self:SetChecked(false)
				if TCF_Settings.debug then
					print("<Show> setChecked(false)") 
				end
			end 
		end)

-- Check button for closing the window 
closebtn = CreateFrame("Button", "TCF_Close", TCF_Options, "OptionsButtonTemplate")
		closebtn:SetWidth(10)
		closebtn:SetHeight(18)
		closebtn:SetPoint("TOPRIGHT", -10, -5)
		closebtn:RegisterForClicks("LeftButtonUp")
		closebtn:SetText("X")
		closebtn:SetNormalFontObject("GameFontNormalSmall")
		closebtn:SetHighlightFontObject("GameFontWhiteSmall")
		closebtn:SetScript("OnClick", function(self)
			self:GetParent():Hide()
		end)
		closebtn:SetScript("OnShow", function(self)
			self:Show()
		end)
		
--   First, a little workaround...apparently there is a known "issue" where the addon sees
--   the same msg repeated SEVEN times in a row.  To work around it, we save the previous
--   msg and if we see that same msg again, ignore it.
local prevmsg = "xxxxxxxxxxxxxxxxxxxxxxx"   -- set to a msg we are unlikely to ever really see

-- Our Filter!  The main point of this Addon!
-- Returning TRUE means filter this message (don't display it)
-- Returning FALSE means don't filter this message (do display it)
local function chatFilterFunction(_,_,msg, sender, lang, chnStr, targ, unkn, chnlNum)
	local filteredstr
	-- If this addon isn't enabled, don't filter anything
	if not TCF_Settings.tradeChatFilter then 
		return false -- don't filter msg
	end
	-- To make comparisons easier, convert msg received to lowercase
	msg = strlower(msg)

	-- Filter out repeated msgs (fix the 7 events in a row issue - see comments above)
	if msg == prevmsg then
		return true -- filter msg
	end
	-- Now that we know we aren't seeing a repeated msg, save the current msg as previous msg
	prevmsg = msg

	-- Check GENERAL and TRADE channel and eliminate all high level (80) LFG, etc messages
	if (chnlNum == 1 or chnlNum == 2) then
		for i = 1,#TCF_TradeFilterList do
			if strfind(msg, TCF_TradeFilterList[i]) then 
				filteredstr = msg
				-- Only display the filtered msg if debug is enabled
				if TCF_Settings.debug then
					DEFAULT_CHAT_FRAME:AddMessage("high level noise eliminated:".."-"..sender.."-"..lang.."-".."-"..filteredstr)
				end
				return true-- filter msg (don't display it)
			end
		end -- for loop

                -- French language filter
		for i = 1,#TCF_FrenchLanguageFilterList do
                    local french = TCF_FrenchLanguageFilterList[i]
                    local search1 = " " .. french .. " "    -- " oui "
                    local search2 = "^" .. french .. " "    -- "^oui "
                    local search3 = " " .. french .. "$"    -- " oui$"
                    -- Cannot simply do strfind on the "french" word because strfind will locate substrs too like "est" in "best"
                    -- So we look for WORDS and do that by putting a leading/trailing space after our search words to compare, etc.
			if ( strfind(msg, search1) or strfind(msg, search2) or strfind(msg, search3) ) then 
				filteredstr = msg
				-- Only display the filtered msg if debug is enabled
				if TCF_Settings.debug then
					DEFAULT_CHAT_FRAME:AddMessage("French language eliminated:".."-"..sender.."-"..lang.."-".."-"..filteredstr)
				end
				return true-- filter msg (don't display it)
			end
		end -- for loop
	end -- if

	-- message passed our filters so go ahead and allow it to be displayed
	return false
end

-- For trade chat filtering, we only need the msg channel as far as I can tell 
local events = {
	"CHAT_MSG_CHANNEL"
	}

-- For each event above, register our filter
--   Kind of silly since we only have one event, but just in case we add more 
--   events in the future let's keep the loop here.
for _,k in ipairs(events) do
	ChatFrame_AddMessageEventFilter(k, chatFilterFunction)
end

-- End of AddOn
